# -*- coding: utf-8 -*-
"""
Module implementing configDialog.
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import os

# Third-party imports
from PyQt5.QtCore import pyqtSlot, Qt, QSize, QDir, QSettings, QCoreApplication, pyqtSignal
from PyQt5.QtWidgets import QDialog, QFileDialog, QMessageBox

# Local application imports
from pycirkuit.ui.Ui_configdialog import Ui_ConfigDialog
from pycirkuit.exceptions import PyCirkuitError
# Translation function
_translate = QCoreApplication.translate

class ConfigDialog(QDialog, Ui_ConfigDialog):
    """
    Class documentation goes here.
    """
    
    exportSettingsChange = pyqtSignal()
    
    def __init__(self, parent=None):
        """
        Constructor
        
        @param parent reference to the parent widget
        @type QWidget
        """
        super().__init__(parent)
        self.setupUi(self)
        
        # Hack to center items in QListView
        for i in range(0, self.listWidget.count()):
            self.listWidget.item(i).setSizeHint(QSize(128, 80))
            self.listWidget.item(i).setTextAlignment(Qt.AlignCenter)
        
        # Persistent settings
        settings = QSettings() 
        
        # Extract stored path to Circuit Macros
        cmStoredPath = settings.value("General/cmPath", "")
        if cmStoredPath == "":
            # emit signal manually as setting an empty text doesn't really changes the text (it is already empty)
            self.cmPath.textChanged.emit("")
        else:
            self.cmPath.setText(cmStoredPath)

        # Extract stored path to LaTeX template file
        # defaultPath = QDir.homePath() + "/Plantilles/cm_tikz.ckt"
        storedLatexTemplateFile = settings.value("General/templatePath", "")
        if storedLatexTemplateFile == "":
            # emit signal manually as setting an empty text doesn't really changes the text (it is already empty)
            self.templateFile.textChanged.emit("")
        else:
            self.templateFile.setText(storedLatexTemplateFile)
            
        # Settings from the second page "Export"
        settings.beginGroup("Export")
        self.exportTIKZ.setChecked(settings.value("exportTIKZ", type=bool))
        self.exportSVG.setChecked(settings.value("exportSVG", type=bool))
        self.exportPDF.setChecked(settings.value("exportPDF", type=bool))
        self.exportPNG.setChecked(settings.value("exportPNG", type=bool))
        self.exportJPEG.setChecked(settings.value("exportJPEG", type=bool))
        self.exportDPI.setValue(settings.value("exportDPI", type=int))
        self.exportQuality.setValue(settings.value("exportQuality", type=int))
        settings.endGroup()

    
    def exportSettingsChanged(self):
        settings = QSettings()
        if (self.exportTIKZ.isChecked() != settings.value("Export/exportTIKZ", type=bool)) or \
           (self.exportSVG.isChecked() != settings.value("Export/exportSVG", type=bool)) or \
           (self.exportPDF.isChecked() != settings.value("Export/exportPDF", type=bool)) or \
           (self.exportPNG.isChecked() != settings.value("Export/exportPNG", type=bool)) or \
           (self.exportJPEG.isChecked() != settings.value("Export/exportJPEG", type=bool)) or \
           (self.exportDPI.value() != settings.value("Export/exportDPI", type=int)) or \
           (self.exportQuality.value() != settings.value("Export/exportQuality", type=int)):
            return True
        else:
            return False
        
        
    @pyqtSlot()
    def accept(self):
        settings = QSettings()
        # General page
        try:
            if not os.path.isdir(self.cmPath.text()):
                message =_translate("MessageBox", "The path to the Circuit Macros location is not valid. Please enter a valid one.", "")
                extraInfo = _translate("MessageBox", "Or, cancel the settings dialog and PyCirkuit will download and install the Circuit Macros when needed.", "")
                raise PyCirkuitError(message, moreInfo=extraInfo)
            if not os.path.isfile(self.templateFile.text()):
                raise PyCirkuitError(_translate("MessageBox", "The path to the LaTeX template does not point to a valid file. Please enter a correct one.", ""))
        except PyCirkuitError as err:
            # Open MessageBox and inform user
            msgBox = QMessageBox(self)
            msgBox.setWindowTitle(err.title)
            msgBox.setIcon(QMessageBox.Critical)
            msgBox.setText(str(err))
            msgBox.setInformativeText(err.moreInfo)
            msgBox.setStandardButtons(QMessageBox.Ok)
            msgBox.setDefaultButton(QMessageBox.Ok)
            msgBox.exec()
            return
        settings.setValue("General/cmPath", self.cmPath.text())
        settings.setValue("General/templatePath", self.templateFile.text())
        # Export page
        if self.exportSettingsChanged():
            QMessageBox.information(self, "PyCirkuit", _translate("MessageBox", "To be able to export with the new settings you have to process the source again.", ""))
            self.exportSettingsChange.emit()
        settings.beginGroup("Export")
        settings.setValue("exportTIKZ", self.exportTIKZ.isChecked())
        settings.setValue("exportSVG", self.exportSVG.isChecked())
        settings.setValue("exportPDF", self.exportPDF.isChecked())
        settings.setValue("exportPNG", self.exportPNG.isChecked())
        settings.setValue("exportJPEG", self.exportJPEG.isChecked())
        settings.setValue("exportDPI", self.exportDPI.value())
        settings.setValue("exportQuality", self.exportQuality.value())
        settings.endGroup()
        # save settings and quit
        settings.sync()
        QDialog.accept(self)

        
    @pyqtSlot()
    def reject(self):
        QDialog.reject(self)
        
        
    @pyqtSlot(int)
    def on_listWidget_currentRowChanged(self, currentRow):
        """
        Slot documentation goes here.
        
        @param currentRow DESCRIPTION
        @type int
        """
        self.stackedWidget.setCurrentIndex(currentRow)
    
    @pyqtSlot()
    def on_toolButtonCMPath_clicked(self):
        """
        Slot documentation goes here.
        """
        fdlg = QFileDialog(self)
        fdlg.setWindowTitle(_translate("ConfigDialog", "Circuit Macros Location", "File Dialog Title"))
        fdlg.setDirectory(self.cmPath.text())
        fdlg.setFileMode(QFileDialog.Directory)
        fdlg.setOptions(QFileDialog.ShowDirsOnly | QFileDialog.DontUseNativeDialog | QFileDialog.ReadOnly)
        fdlg.setViewMode(QFileDialog.Detail)
        fdlg.setFilter(QDir.Dirs | QDir.Hidden)
        if fdlg.exec():
            newPath = fdlg.selectedFiles()
            self.cmPath.setText(newPath[0])
        fdlg.close()

    
    @pyqtSlot()
    def on_toolButtonTemplatePath_clicked(self):
        """
        Slot documentation goes here.
        """
        fdlg = QFileDialog(self)
        fdlg.setWindowTitle(_translate("ConfigDialog", "Latex Template Location", "File Dialog Title"))
        fdlg.setDirectory(os.path.dirname(self.templateFile.text()))
        fdlg.setFileMode(QFileDialog.ExistingFile)
        fdlg.setOptions(QFileDialog.DontUseNativeDialog | QFileDialog.ReadOnly)
        fdlg.setViewMode(QFileDialog.Detail)
        fdlg.setFilter(QDir.AllDirs | QDir.Files | QDir.NoDotAndDotDot | QDir.Hidden)
        if fdlg.exec():
            newPath = fdlg.selectedFiles()
            self.templateFile.setText(newPath[0])
        fdlg.close()


    @pyqtSlot(str)
    def on_cmPath_textChanged(self, newText):
        if  os.path.isdir(newText):
            self.cmPath.setStyleSheet("background-color: {white};".format(white = "rgb(255, 255, 255)"))
        else:
            self.cmPath.setStyleSheet("background-color: {reddish};".format(reddish = "rgb(255, 230, 230)"))


    @pyqtSlot(str)
    def on_templateFile_textChanged(self, newText):
        if  os.path.isfile(newText):
            self.templateFile.setStyleSheet("background-color: {white};".format(white = "rgb(255, 255, 255)"))
        else:
            self.templateFile.setStyleSheet("background-color: {reddish};".format(reddish = "rgb(255, 230, 230)"))



# -*- coding: utf-8 -*-
"""
Module implementing the PyCirkuit exceptions
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Third-party imports
from PyQt5.QtCore import QCoreApplication

# Translation function
_translate = QCoreApplication.translate

# Exceptions in pycirkuit are used mainly to display info to the user via MessageBoxes
# Thus, an exception must convey 3 infos:
# 1) The MessageBox Title, different for each kind of error
# 2) The main message, perhaps as short as possible
# 3) Additional info
class PyCirkuitError(Exception):
    def __init__(self, message, title="", moreInfo=""):
        super().__init__(message)
        if title == "":
            self.title = _translate("PyCirkuitError", "PyCirkuit Error", "Exception title")
        else:
            self.title = title
        self.moreInfo=moreInfo


# Image manipulation exceptions
class PyCktImageError(PyCirkuitError):
    def __init__(self):
        # Define error message, title (if different from default) and moreInfo (if different from default)
        # call parent constructor with these 3 strings as params.
        errMsg = _translate("PyCirkuitError", "The generated image could not be loaded.",  "Exception message")
        super().__init__(errMsg)


# Tool Doc exceptions
class PyCktDocNotFoundError(PyCirkuitError):
    def __init__(self, toolName):
        errMsg = _translate("ExternalTool", "Cannot find the {toolName} manual!", "Leave untranslated the variable name inside curly braces (included)")
        errMsg = errMsg.format(toolName=toolName)
        info = _translate("ExternalTool", "Please ensure that you have this application properly installed, with its documentation into one of these standard locations:\n\n")
        super().__init__(errMsg, title=_translate("ExternalTool", "File Not Found", "Exception title"), moreInfo=info)

class PyCktCMManNotFoundError(PyCirkuitError):
    def __init__(self, cmPath):
        errMsg  = _translate("ExternalTool", 'Cannot find the "Circuit Macros" documentation.\n\n', "Warning message")
        errMsg += _translate("ExternalTool", "You will have to search for it manually. It should be a PDF file located into {cmPath} folder or one of its subfolders.", "Message Box text. DO NOT translate '{cmPath}' variable.").format(cmPath=cmPath)
        super().__init__(errMsg, title=_translate("ExternalTool", "File Not Found", "Exception title"), moreInfo="")


# Tool exceptions
class PyCktToolExecutionError(PyCirkuitError):
    def __init__(self, message, moreInfo="", tool=""):
        super().__init__(message, title=_translate("ExternalTool", "Tool Execution Error", "Exception title"), moreInfo=moreInfo)
        self.tool = tool

class PyCktToolNotFoundError(PyCirkuitError):
    def __init__(self, executableName, longName):
        errMsg = _translate("ExternalTool", "Cannot find the {toolLongName}!\n\n", "Leave untranslated the variable name inside curly braces (included)")
        errMsg = errMsg.format(toolLongName=longName)
        info = _translate("ExternalTool", "Please ensure that you have this application properly installed and the executable \"{toolExecutableName}\" is in the PATH.\n\n", "Leave untranslated the variable name inside curly braces (included)")
        info += _translate("ExternalTool", "Cannot generate the preview.")
        info = info.format(toolExecutableName=executableName)
        super().__init__(errMsg, title=_translate("ExternalTool", "Tool Not Found", "Exception title"), moreInfo=info)


# Circuit Macros exceptions
class PyCktCMNotFoundError(PyCirkuitError):
    def __init__(self, message):
        super().__init__(message, title=_translate("ExternalTool", "Circuit Macros not found", "Exception title"))

class PyCktCMNewVersionAvailable(PyCirkuitError):
    def __init__(self, message):
        super().__init__(message, title=_translate("ExternalTool", "New Circuit Macros version available!", "Exception title"))

class PyCktCMFetchError(PyCirkuitError):
    def __init__(self, message):
        super().__init__(message, title=_translate("ExternalTool", "Circuit Macros not found", "Exception title"))


# Other exceptions
class PyCktLatexTemplateError(PyCirkuitError):
    def __init__(self, message):
        super().__init__(message, title="")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/orestes/Devel/Software/pycirkuit/pycirkuit/ui/mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(677, 667)
        MainWindow.setMinimumSize(QtCore.QSize(440, 400))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/AppIcon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        font = QtGui.QFont()
        font.setKerning(True)
        self.tabWidget.setFont(font)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_source = QtWidgets.QWidget()
        self.tab_source.setObjectName("tab_source")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tab_source)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.sourceText = pycktTextEditor(self.tab_source)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceText.sizePolicy().hasHeightForWidth())
        self.sourceText.setSizePolicy(sizePolicy)
        self.sourceText.setObjectName("sourceText")
        self.horizontalLayout_3.addWidget(self.sourceText)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/Text"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab_source, icon1, "")
        self.tab_log = QtWidgets.QWidget()
        self.tab_log.setObjectName("tab_log")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tab_log)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.outputText = QtWidgets.QPlainTextEdit(self.tab_log)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        font.setPointSize(11)
        self.outputText.setFont(font)
        self.outputText.setReadOnly(True)
        self.outputText.setPlainText("")
        self.outputText.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.outputText.setObjectName("outputText")
        self.horizontalLayout_4.addWidget(self.outputText)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/Output"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab_log, icon2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.processButton = QtWidgets.QPushButton(self.centralwidget)
        self.processButton.setEnabled(False)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/Run"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.processButton.setIcon(icon3)
        self.processButton.setObjectName("processButton")
        self.horizontalLayout.addWidget(self.processButton)
        self.exportButton = QtWidgets.QPushButton(self.centralwidget)
        self.exportButton.setEnabled(False)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/Export"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.exportButton.setIcon(icon4)
        self.exportButton.setObjectName("exportButton")
        self.horizontalLayout.addWidget(self.exportButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 677, 30))
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtWidgets.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuSettings = QtWidgets.QMenu(self.menuBar)
        self.menuSettings.setObjectName("menuSettings")
        MainWindow.setMenuBar(self.menuBar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.previewWidget = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.previewWidget.sizePolicy().hasHeightForWidth())
        self.previewWidget.setSizePolicy(sizePolicy)
        self.previewWidget.setMinimumSize(QtCore.QSize(100, 130))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/view-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.previewWidget.setWindowIcon(icon5)
        self.previewWidget.setObjectName("previewWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidgetContents.sizePolicy().hasHeightForWidth())
        self.dockWidgetContents.setSizePolicy(sizePolicy)
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.imageViewer = pycktImageViewer(self.dockWidgetContents)
        self.imageViewer.setAcceptDrops(False)
        self.imageViewer.setAutoFillBackground(True)
        self.imageViewer.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.imageViewer.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)
        self.imageViewer.setResizeAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
        self.imageViewer.setViewportUpdateMode(QtWidgets.QGraphicsView.FullViewportUpdate)
        self.imageViewer.setRubberBandSelectionMode(QtCore.Qt.IntersectsItemShape)
        self.imageViewer.setObjectName("imageViewer")
        self.horizontalLayout_2.addWidget(self.imageViewer)
        self.previewWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(4), self.previewWidget)
        self.actionQuit = QtWidgets.QAction(MainWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/Exit"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionQuit.setIcon(icon6)
        self.actionQuit.setObjectName("actionQuit")
        self.actionOpen = QtWidgets.QAction(MainWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/Open"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpen.setIcon(icon7)
        self.actionOpen.setObjectName("actionOpen")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/icons/About"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAbout.setIcon(icon8)
        self.actionAbout.setObjectName("actionAbout")
        self.actionPreferences = QtWidgets.QAction(MainWindow)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/icons/Settings"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPreferences.setIcon(icon9)
        self.actionPreferences.setObjectName("actionPreferences")
        self.actionNew = QtWidgets.QAction(MainWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/icons/New"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionNew.setIcon(icon10)
        self.actionNew.setObjectName("actionNew")
        self.actionSave = QtWidgets.QAction(MainWindow)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/icons/Save"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSave.setIcon(icon11)
        self.actionSave.setObjectName("actionSave")
        self.actionSaveAs = QtWidgets.QAction(MainWindow)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/icons/SaveAs"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSaveAs.setIcon(icon12)
        self.actionSaveAs.setObjectName("actionSaveAs")
        self.actionCMMan = QtWidgets.QAction(MainWindow)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/icons/CMman"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCMMan.setIcon(icon13)
        self.actionCMMan.setObjectName("actionCMMan")
        self.actionDpicMan = QtWidgets.QAction(MainWindow)
        self.actionDpicMan.setIcon(icon13)
        self.actionDpicMan.setObjectName("actionDpicMan")
        self.menuFile.addAction(self.actionNew)
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionSaveAs)
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionCMMan)
        self.menuHelp.addAction(self.actionDpicMan)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menuSettings.addAction(self.actionPreferences)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menuSettings.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        self.actionQuit.triggered.connect(MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "PyCirkuit - by Orestes Mas"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_source), _translate("MainWindow", "Source Editor", "Tab title"))
        self.outputText.setPlaceholderText(_translate("MainWindow", "<Empty output>", "Placeholder text"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_log), _translate("MainWindow", "Output Log", "Tab title"))
        self.processButton.setText(_translate("MainWindow", "&Process and display", "Button text"))
        self.processButton.setShortcut(_translate("MainWindow", "Alt+P"))
        self.exportButton.setText(_translate("MainWindow", "E&xport", "Button text"))
        self.exportButton.setShortcut(_translate("MainWindow", "Alt+X"))
        self.menuFile.setTitle(_translate("MainWindow", "Fi&le"))
        self.menuHelp.setTitle(_translate("MainWindow", "Hel&p"))
        self.menuSettings.setTitle(_translate("MainWindow", "Setti&ngs"))
        self.previewWidget.setWindowTitle(_translate("MainWindow", "Preview", "A window title, NOT an action."))
        self.imageViewer.setToolTip(_translate("MainWindow", "<html><head/><body><p>Use:</p>\n"
"<ul>\n"
"<li> Ctrl + Mouse Wheel to Zoom In/Out</li>\n"
"<li> Click and Move to drag the image inside the preview</li>\n"
"</ul>\n"
"</body></html>"))
        self.actionQuit.setText(_translate("MainWindow", "&Quit", "Menu item"))
        self.actionQuit.setStatusTip(_translate("MainWindow", "Close application", "Status Bar Message"))
        self.actionQuit.setShortcut(_translate("MainWindow", "Ctrl+Q", "Keyboard Shortcut"))
        self.actionOpen.setText(_translate("MainWindow", "&Open...", "Menu item"))
        self.actionOpen.setToolTip(_translate("MainWindow", "Open Drawing"))
        self.actionOpen.setStatusTip(_translate("MainWindow", "Opens an existing drawing", "Status Bar Message"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O", "Keyboard Shortcut"))
        self.actionAbout.setText(_translate("MainWindow", "&About PyCirkuit", "Menu item"))
        self.actionAbout.setStatusTip(_translate("MainWindow", "Application information", "Status Bar Message"))
        self.actionPreferences.setText(_translate("MainWindow", "Configure &PyCirkuit...", "Menu item"))
        self.actionPreferences.setStatusTip(_translate("MainWindow", "Opens config dialog", "Status Bar Message"))
        self.actionPreferences.setShortcut(_translate("MainWindow", "Ctrl+P", "Keyboard Shortcut"))
        self.actionNew.setText(_translate("MainWindow", "&New", "Menu item"))
        self.actionNew.setToolTip(_translate("MainWindow", "New Drawing"))
        self.actionNew.setStatusTip(_translate("MainWindow", "Creates an empty drawing", "Status Bar Message"))
        self.actionNew.setShortcut(_translate("MainWindow", "Ctrl+N", "Keyboard Shortcut"))
        self.actionSave.setText(_translate("MainWindow", "&Save", "Menu item"))
        self.actionSave.setStatusTip(_translate("MainWindow", "Saves drawing", "Status Bar Message"))
        self.actionSave.setShortcut(_translate("MainWindow", "Ctrl+S", "Keyboard Shortcut"))
        self.actionSaveAs.setText(_translate("MainWindow", "Sa&ve As...", "Menu item"))
        self.actionSaveAs.setStatusTip(_translate("MainWindow", "Saves drawing with new name", "Status Bar Message"))
        self.actionSaveAs.setShortcut(_translate("MainWindow", "Ctrl+Shift+S", "Keyboard Shortcut"))
        self.actionCMMan.setText(_translate("MainWindow", "&Circuit Macros manual"))
        self.actionCMMan.setStatusTip(_translate("MainWindow", "Opens Circuit Macros manual", "Status Bar Message"))
        self.actionCMMan.setShortcut(_translate("MainWindow", "Ctrl+M"))
        self.actionDpicMan.setText(_translate("MainWindow", "&Dpic manual"))
        self.actionDpicMan.setStatusTip(_translate("MainWindow", "Opens Dpic manual", "Status Bar Message"))
        self.actionDpicMan.setShortcut(_translate("MainWindow", "Ctrl+D"))
from pycirkuit.imageviewer import pycktImageViewer
from pycirkuit.texteditor import pycktTextEditor
from pycirkuit.resources import resources_rc


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
Module implementing an abstract class to handle external tools
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import abc
import os
import subprocess
import inspect
import platform

# Third-party imports
from PyQt5.QtCore import QCoreApplication

# Local application imports
import pycirkuit
from pycirkuit.exceptions import *

# Translation function
_translate = QCoreApplication.translate

# Base class for external tools
class ExternalTool(abc.ABC):
    # Class variable
    ID = ""
    def __init__(self, executableName, longName):
        self.longName = longName
        execPath = os.get_exec_path()

        # Windows-specific
        if platform.system() == 'Windows':
            # Append '.exe' to the executable name we're searching
            executableName = executableName + '.exe'
            # Also, we add an entry to the executable search path pointing to our "lib" dir
            libDir = os.path.dirname(inspect.getfile(pycirkuit))
            libDir = os.path.join(libDir, 'lib')
            execPath.append(libDir)
 
        for testPath in execPath:
            p = os.path.join(testPath, "{execName}".format(execName=executableName))
            if os.path.exists(p):
                self.execPath = testPath
                self.executableName = os.path.join(testPath,  executableName)
                return
        else:
            raise PyCktToolNotFoundError(executableName, self.longName)

    @abc.abstractmethod
    def execute(self, cmd, errMsg, destination=None):
        try:
            # Invoke external tool to do the job
            # For tools that give their output to STDOUT, we capture it and write contents to a file later
            # TODO: Python Doc says "Note: Do not use stdout=PIPE or stderr=PIPE with this function as that can deadlock based on the child process output volume. Use Popen with the communicate() method when you need pipes. "
            result = subprocess.run(cmd, shell=False, check=False, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
            if result.returncode != 0:
                info = result.stderr.decode()
                # By calling self.ID we use polymorphism here
                raise PyCktToolExecutionError(errMsg, moreInfo=info, tool=self.__class__)
            else:
                if destination != None:
                    with open(destination, 'wb') as tmpFile:
                        tmpFile.write(result.stdout)
        except PyCktToolExecutionError as err:
            raise err



#!/usr/bin/python3
# coding: utf-8
"""
Main program entry point/function
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import sys, os, inspect, platform
from os.path import isfile

# Third-party imports
from PyQt5.QtCore import QCoreApplication, \
                         QTranslator, \
                         QLocale, \
                         QLibraryInfo, \
                         QSettings
from PyQt5.QtWidgets import QApplication

# Local application imports
import pycirkuit
from pycirkuit.tools.cliparser import PyCirkuitParser
from pycirkuit.ui.mainwindow import MainWindow

# Resources for translation
from pycirkuit.resources import resources_rc
# Translation function
_translate = QCoreApplication.translate

# Function to ensure meaningful settings
def _check_settings():
    """
    Check if config file is empty (deleted, first run, etc.) and if so, set reasonable default values
    """
    settings = QSettings()
    # Start updating the "General" section
    settings.beginGroup("General")
    # Import ourselves
    import pycirkuit
    # Find absolute application's path
    applicationPath = os.path.dirname(inspect.getfile(pycirkuit))
    # Check app version
    storedVersion = settings.value("Version", "")
    if (storedVersion == ""):
        settings.setValue("Version", pycirkuit.__version__)
    elif (storedVersion < pycirkuit.__version__):
        #TODO: If upgrading we can remove the unnecessary entries, rename others, etc.
        pass
    elif (storedVersion > pycirkuit.__version__):
        #TODO: Handle downgrading, perhaps raising an exception, showing warning, etc.
        pass
    # Check the stored path to LaTeX templates
    if (settings.value("templatePath",  "") == ""):
        # Add the relative path where the default template is located
        templatePath = os.path.normpath(os.path.join(applicationPath, 'templates/cm_tikz.tpl'))
        settings.setValue("templatePath", templatePath)
    # Check Circuit Macros path
    from pycirkuit.tools.circuitmacrosmanager import CircuitMacrosManager
    if (settings.value("cmPath",  "") == ""):
        CM = CircuitMacrosManager()
        settings.setValue("cmPath", CM.default_CMPath())
    # Check the stored path to the built-in examples
    if (settings.value("examplesPath",  "") == ""):
        # Add the relative path where the examples are located
        examplesPath = os.path.normpath(os.path.join(applicationPath, 'examples'))
        settings.setValue("examplesPath", examplesPath)
    # Check last working dir (from where the files to be opened are taken)
    # This one will be initially the same as the examples path. Later on, the user actions will change it.
    if (settings.value("lastWD",  "") == ""):
        # Add the relative path where the examples are located
        examplesPath = os.path.normpath(os.path.join(applicationPath, 'examples'))
        settings.setValue("lastWD", examplesPath)
    # Check the stored path to built-in documentation
    if (settings.value("docPath",  "") == ""):
        # Add the relative path where the examples are located
        docPath = os.path.normpath(os.path.join(applicationPath, 'doc'))
        settings.setValue("docPath", docPath)
    settings.endGroup()   # Finished updating the "General" section
    # Begin updating the "Export" section
    settings.beginGroup("Export")
    settings.setValue("exportTIKZ", settings.value("exportTIKZ", True, type=bool))
    settings.setValue("exportPDF", settings.value("exportPDF", False, type=bool))
    settings.setValue("exportPNG", settings.value("exportPNG", False, type=bool))
    settings.setValue("exportJPEG", settings.value("exportJPEG", False, type=bool))
    settings.setValue("exportDPI", settings.value("exportDPI", 150, type=int))
    settings.setValue("exportQuality", settings.value("exportQuality", 80, type=int))
    settings.endGroup()  # Finished updating the "Export" section
    
    # Save settings for other object's use
    settings.sync()

# Main entry point
def main():
    if ("DESKTOP_SESSION" in os.environ) or (platform.system() == 'Windows') or (platform.system() == 'Darwin'):
        app = QApplication(sys.argv)
        pycirkuit.__haveGUI__ = True
    else:
        app = QCoreApplication(sys.argv)
        pycirkuit.__haveGUI__ = False
  
    # First try to load the Qt-provided translations (used in some standard dialog strings)
    qtTranslator = QTranslator()
    filename = "qtbase_" + QLocale.system().name().split("_")[0]
    if qtTranslator.load(filename, QLibraryInfo.location(QLibraryInfo.TranslationsPath), suffix=".qm"):
        app.installTranslator(qtTranslator)
    # Then load PyCirkuit translations
    pycirkuitTranslator = QTranslator()
    if pycirkuitTranslator.load(QLocale(), "pycirkuit", ".", ":/translations", ".qm"):
        app.installTranslator(pycirkuitTranslator)
        
    # These two next values are passed to every instance of QSettings everywhere in the app
    QCoreApplication.setOrganizationName("PyCirkuit")
    QCoreApplication.setApplicationName("pycirkuit")
    QCoreApplication.setApplicationVersion(pycirkuit.__version__)

    # Ensure meaningful settings
    _check_settings()
    
    # Parse command line options, deciding if user requested to work interactively (GUI)
    # or to process a list of files in batch mode (CLI)
    # If the former is chosen, this call returns with an eventual file to open in to the GUI
    # Otherwise the call don't return and does the process itself.
    cmdLineParser = PyCirkuitParser(sys.argv)
    fileToOpen = cmdLineParser.parseCmdLine()

    # Start GUI
    if not isinstance(app, QApplication):
        sys.exit(_translate("CommandLine", "No desktop environment available.", ""))
    my_mainWindow = MainWindow()
    my_mainWindow.show()
    if fileToOpen != None:
        if isfile(fileToOpen):
            my_mainWindow._load_file(fileToOpen)
        else:
            my_mainWindow.on_actionNew_triggered(fileToOpen)
    app.exec()
    
    
if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Module implementing a CircuitMacros Manager class.
"""
# Copyright (C) 2018-2019 Orestes Mas
# Copyright (C) 2019 Aniol Marti
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import os
from os.path import normpath, join, exists, dirname, abspath
import zipfile
import urllib.request as Net
import urllib.error as NetError

# Third-party imports
from PyQt5 import QtCore
from PyQt5.QtCore import QStandardPaths

# Local application imports
from pycirkuit.exceptions import *

# Translation function
_translate = QtCore.QCoreApplication.translate

class CircuitMacrosManager(QtCore.QObject):
    """
    Class documentation goes here.
    """
    CIRCUIT_MACROS_FILE = "circuit_macros.zip"
    def __init__(self, parent=None):
        """
        Constructor

        @param parent reference to the parent widget
        @type QWidget
        """
        pass

    def check_installed(self):
        #TODO: Should:
        #  1) Test several places, starting with the one set up in config, but then test also standard locations
        #  2) Two possibilities can arise:
        #     a) There's only one place where CM are
        #     b) They are found in two or more places
        dirList =  QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
        # Append specific app subdir to each possible location
        dirList = [normpath(join(dir, "circuit-macros")) for dir in dirList]
        settings = QtCore.QSettings()
        extraPath = normpath(settings.value("General/cmPath", ""))
        if extraPath != "":
            dirList.insert(0, extraPath)
        for testPath in dirList:
            if exists(join(testPath , "libcct.m4")):
                #  3) We pick up the first found. Perhaps we might consider other possibilities
                #  4) Finally, as a SIDE EFFECT we can update config automatically or not.
                settings.setValue("General/cmPath", testPath)
                return True
        else:
            return False

    def default_CMPath(self):
        return normpath(join(QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.AppDataLocation), "circuit-macros"))
        
    def download_latest(self, percent):
        origin = "https://ctan.mirror.norbert-ruehl.de/graphics/" + CircuitMacrosManager.CIRCUIT_MACROS_FILE
        destination = dirname(self.default_CMPath())
        if destination == "":
            raise PyCirkuitError(_translate("ExternalTool", "Cannot determine the standard writable location for PyCirkuit",  "Error message"))
        if not exists(destination):
            os.makedirs(destination)
        destination = join(destination, CircuitMacrosManager.CIRCUIT_MACROS_FILE)
        try:
            with Net.urlopen(origin) as source,  open(destination, 'wb') as dest:
                length = source.getheader('content-length')
                if length:
                    length = int(length)
                    percent.setRange(0, length)
                    percent.setValue(0)
                    blocksize = max(4096, length//100)
                else:
                    blocksize = 1000000 # just made something up
                size = 0
                while True:
                    buf1 = source.read(blocksize)
                    if not buf1:
                        break
                    dest.write(buf1)
                    size += len(buf1)
                    percent.setValue(size)
        except NetError.URLError as e:
            #FIXME: Convert to MessageBox by reraising as PyCktCMFetchError
            print(_translate("ExternalTool", "Network error: ",  "Error message"), e)

    def getManUrl(self):
        import glob
        try:
            import magic
        except ImportError:
            raise PyCirkuitError(_translate("ExternalTool", "Module 'python-magic' not found. Please check that all PyCirkuit dependencies are correctly installed.",  "Error message"))
        mime = magic.Magic(mime=True)   # Prepare to detect a file's mimetype based on its contents
        # Get standard locations for documentation in a platform-independent way
        dirList = QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
        # Append specific app subdir to each possible location
        dirList = [normpath(join(dir, "doc", "circuit-macros")) for dir in dirList]
        # Add the config-stored Circuit Macros Location to this list
        settings = QtCore.QSettings()
        extraPath = normpath(settings.value("General/cmPath",  ""))
        extraPath = join(extraPath, "doc")
        dirList.insert(0, extraPath)
        # Add the default Circuit Macros location to this list (can be the same as above)
        dirList.append(join(self.default_CMPath(), "doc"))
        # Explore the generated list searching for pdf or compressed pdf
        for testPath in dirList:
            # Perhaps we should search for *.pdf* or, at least, for *.pdf AND *.pdf.gz
            candidates = glob.glob(join(testPath, "Circuit_macros.pdf"))
            candidates.extend(glob.glob(join(testPath, "Circuit_macros.pdf.gz")))
            for candidate in candidates:
                mimeType = mime.from_file(candidate)
                if (mimeType == "application/pdf") or (mimeType == "application/gzip"):
                    return(candidate)
        else:
            raise PyCktCMManNotFoundError(normpath(settings.value("General/cmPath",  "")))

    def unpack_circuit_macros(self):
        try:
            dataPath = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.AppDataLocation)
            zipName = abspath(join(dataPath,  CircuitMacrosManager.CIRCUIT_MACROS_FILE))
            zip_ref = zipfile.ZipFile(zipName, "r")
            zip_parent_dir = zip_ref.namelist()[0]
            zip_ref.extractall(dataPath)
            zip_ref.close()
            os.rename(join(dataPath, zip_parent_dir), join(dataPath, "circuit-macros"))
            os.remove(zipName)
            settings = QtCore.QSettings()
            settings.setValue("General/cmPath", join(dataPath , 'circuit-macros'))
            settings.sync()
        except Exception as e:
            #FIXME: Convert to MessageBox by reraising as PyCktCMFetchError
            print(_translate("ExternalTool", "Error uncompressing the Circuit Macros: ",  "Error message"), e)



"""
Copyright (c) 2018 Orestes Mas

This file is part of PyCirkuit.

PyCirkuit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PyCirkuit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
"""

from setuptools import setup, find_packages
import pycirkuit
import platform

with open("README.md", "r") as fh:
    long_description = fh.read()
    
setup(name = 'pycirkuit',
    version = pycirkuit.__version__,
    description = pycirkuit.__description__, 
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = pycirkuit.__homepage__,
    author = pycirkuit.__author__,
    author_email = pycirkuit.__author_email__,
    license = pycirkuit.__license_short__,
    packages = find_packages(),
    package_data = {
        'pycirkuit': ['doc/*', 'lib/*', 'templates/*', 'examples/*'],
    },
    entry_points = {
        'gui_scripts': [
            'pycirkuit = pycirkuit.main:main',
        ],
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Topic :: Multimedia :: Graphics :: Editors",
        "Topic :: Scientific/Engineering",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Intended Audience :: End Users/Desktop",
    ],
    install_requires = [
        # Debian package is python3-pyqt5
        'PyQt5',
        'wheel',
        # Debian package is python3-magic
        'python-magic-bin' if platform.system() == 'Windows' else 'python-magic'
    ],
    zip_safe = True)



# -*- coding: utf-8 -*-
"""
Module implementing MainWindow.
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
#import sys
import os
from shutil import copyfile

# Third-party imports
from PyQt5.QtCore import pyqtSlot, Qt, QCoreApplication, QSettings
from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtWidgets import QProgressBar

# Local application imports
from pycirkuit.ui.Ui_mainwindow import Ui_MainWindow
from pycirkuit.ui.configdialog import ConfigDialog
from pycirkuit.ui.aboutdialog import AboutDialog
from pycirkuit.tools.circuitmacrosmanager import CircuitMacrosManager
from pycirkuit.highlighter import PyCirkuitHighlighter
from pycirkuit.exceptions import *
from pycirkuit.tools.m4 import ToolM4
from pycirkuit.tools.dpic import ToolDpic
from pycirkuit.tools.pdflatex import ToolPdfLaTeX
from pycirkuit.tools.pdftopng import ToolPdfToPng
from pycirkuit.tools.pdftojpg import ToolPdfToJpeg
from pycirkuit.tools.processor import PyCirkuitProcessor
import pycirkuit


# Translation function
_translate = QCoreApplication.translate

class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    """
    Class documentation goes here.
    """
    def __init__(self, parent=None):
        """
        Constructor

        @param parent reference to the parent widget
        @type QWidget
        """
        super().__init__(parent)
        # This is the translated name for unnamed files
        self._translatedUnnamed = _translate("MainWindow", "unnamed", "Initial name of a new empty file")
        
        # This is to avoid starting the app with buffer marked as "dirty", and hence needing saving
        # This occurs because the "setupUI" method modifies text and hence triggers a textChanged signal
        self.insideConstructor = True
        self.setupUi(self)

        # The app icon is located inside the resource file
        icon = QtGui.QIcon(":/icons/AppIcon")
        self.setWindowIcon(icon)

        # Connect signals with slots
        #NOTE: Is NOT necessary to MANUALLY connect most signals to slots, as 
        # pyuic5 calls QtCore.QMetaObject.connectSlotsByName in Ui_configdialog.py
        # do such connections AUTOMATICALLY (so connecting them manually triggers slots twice)
        self.imageViewer.conversion_failed.connect(self._display_error)
        self.imageViewer.image_changed.connect(self._resize_preview)
        self.previewWidget.dockLocationChanged.connect(self._resize_preview)

        # Instantiate a processor object which will perform the actual file processing
        self.processor = PyCirkuitProcessor()

        # Set up the editor
        font = QtGui.QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(12)
        self.sourceText.setFont(font)
        self.highlighter = PyCirkuitHighlighter(self.sourceText.document())
        # Initialize editor contents with a default drawing template
        self.needSaving = False
        self.on_actionNew_triggered()
        
        # Center the window on screen
        self._center()

        # Set up the permanent widgets in status bar
        self.sbProgressBar = QProgressBar()
        self.sbProgressBar.setRange(0, 100)
        self.sbProgressBar.setMaximumHeight(10)
        self.sbProgressBar.setTextVisible(False)
        self.sbProgressBar.setVisible(False)
        self.statusBar.addPermanentWidget(self.sbProgressBar)
        
        # We're quitting constructor
        self.insideConstructor = False

    def _ask_export_as(self, src, dst):
        fdlg = QtWidgets.QFileDialog(self)
        fdlg.setWindowTitle(_translate("MainWindow", "Enter a file to save into",  "Window Title"))
        fdlg.setDirectory(dst)
        fdlg.setFileMode(QtWidgets.QFileDialog.AnyFile)
        fdlg.setOptions(QtWidgets.QFileDialog.DontUseNativeDialog)
        fdlg.setViewMode(QtWidgets.QFileDialog.Detail)
        fdlg.setFilter(QtCore.QDir.AllDirs | QtCore.QDir.Files)
        fdlg.setNameFilters([
            _translate("MainWindow", "TikZ files (*.tikz)", "File filter"),
            _translate("MainWindow", "PDF files (*.pdf)", "File filter"),
            _translate("MainWindow", "PNG files (*.png)", "File filter"),
            _translate("MainWindow", "JPEG files (*.jpeg *.jpg)", "File filter"),
            _translate("MainWindow", "TeX files (*.tex)", "File filter"),
            _translate("MainWindow", "Any files (*)", "File filter")])
        if fdlg.exec():
            dst = fdlg.selectedFiles()[0]
            copyfile(src, dst)
            return True
        fdlg.close()
        return False


    def _ask_saving(self):
        # Open MessageBox and inform user
        msgBox = QtWidgets.QMessageBox(self)
        msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Warning",  "Message Box title"))
        msgBox.setIcon(QtWidgets.QMessageBox.Warning)
        msgBox.setText(_translate("MessageBox", "Source file have unsaved changes.", "Message box text"))
        msgBox.setInformativeText(_translate("MessageBox", "Do you want to save them before proceeding?",  "Message Box text"))
        msgBox.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
        msgBox.setDefaultButton(QtWidgets.QMessageBox.Yes)
        response = msgBox.exec()
        if response == QtWidgets.QMessageBox.Yes:
            self.actionSave.trigger()


    def _center(self):
        # Get desktop's geometry
        app = QtWidgets.QApplication.instance()
        desktop = app.desktop()
        deskRect = desktop.screenGeometry(-1)
        # Now get MainWindow's geometry
        winRect = self.frameGeometry()
        # Now translate the window's rectangle so its center coincides with the desktop's one
        winRect.moveCenter(deskRect.center())
        # Finally, move the physical window using the top-left corner coords:
        self.move(winRect.x(), winRect.y())


    def _check_programs(self):
        try:
            # Dictionary using a class as index and a class instance as value
            self.extTools = {
                ToolM4: ToolM4(),
                ToolDpic: ToolDpic(), 
                ToolPdfLaTeX: ToolPdfLaTeX(), 
                ToolPdfToPng: ToolPdfToPng(), 
                ToolPdfToJpeg: ToolPdfToJpeg()
            }
        except PyCktToolNotFoundError as err:
            # Open MessageBox and inform user
            msgBox = QtWidgets.QMessageBox(self)
            msgBox.setWindowTitle(err.title)
            msgBox.setIcon(QtWidgets.QMessageBox.Critical)
            msgBox.setText(str(err))
            msgBox.setInformativeText(err.moreInfo)
            msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
            msgBox.setDefaultButton(QtWidgets.QMessageBox.Ok)
            msgBox.exec()
            return False
        return True


    def _check_templates(self):
        settings = QSettings()
        template = settings.value("General/templatePath",  "")
        if os.path.exists(template):
            with open(template, 'r') as t:
                templateCode = t.read()
                if "%%SOURCE%%" in templateCode:
                    return True
                else:
                    errMsg  = _translate("MessageBox", "The specified LaTeX template seems invalid!\n\n", "Error message")
                    errMsg += _translate("MessageBox", "Please indicate a correct one in the Settings.\n\n", "Error message")
                    errMsg += _translate("MessageBox", "Cannot generate the preview.", "Error message")
                    error = PyCirkuitError(errMsg)
                    self._display_error(error)
                    return False
        else:
            errMsg  = _translate("MessageBox", "The LaTeX template has not been found!\n\n", "Error message")
            errMsg += _translate("MessageBox", "Please indicate its correct PATH in the Settings.\n\n", "Error message")
            errMsg += _translate("MessageBox", "Cannot generate the preview.", "Error message")
            error = PyCirkuitError(errMsg)
            self._display_error(error)
            return False


    @pyqtSlot(PyCirkuitError)
    def _display_error(self, error):
        # Open MessageBox and inform user
        #TODO: Use this method to display message boxes
        msgBox = QtWidgets.QMessageBox(self)
        msgBox.setWindowTitle(error.title)
        msgBox.setIcon(QtWidgets.QMessageBox.Critical)
        msgBox.setText(str(error))
        msgBox.setInformativeText(error.moreInfo)
        msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
        msgBox.setDefaultButton(QtWidgets.QMessageBox.Ok)
        msgBox.exec()


    def _enforce_circuit_macros(self):
        cmMgr = CircuitMacrosManager(self)
        if cmMgr.check_installed():
            return True
        else:
            _cmNotFound  = _translate("MessageBox", "Cannot find the 'Circuit Macros'!\n\n")
            txt = _cmNotFound + _translate("MessageBox", "Do you want to try to search and install them automatically?")
            response = QtWidgets.QMessageBox.question(self, _translate("MessageBox", "PyCirkuit - Warning", "Message Box title"),  txt,  defaultButton=QtWidgets.QMessageBox.Yes)
            result = False
            if response == QtWidgets.QMessageBox.Yes:
                try:
                    self.statusBar.showMessage(_translate("StatusBar", "Downloading and unpacking Circuit Macros", "Status Bar message"))
                    app = QtWidgets.QApplication.instance()
                    app.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
                    self.sbProgressBar.setRange(0, 100)
                    self.sbProgressBar.setValue(0)
                    self.sbProgressBar.setVisible(True)
                    cmMgr.download_latest(self.sbProgressBar)
                    cmMgr.unpack_circuit_macros()
                    result = True
                except:
                    pass
                finally:
                    app.restoreOverrideCursor()
                    self.statusBar.clearMessage()
                    self.sbProgressBar.setVisible(False)
            else:
                txt = _cmNotFound + _translate("MessageBox", "Please indicate the correct path to them in the settings dialog.")
                QtWidgets.QMessageBox.critical(self, _translate("MessageBox", "PyCirkuit - Error", "Message Box title"),  txt)
            return result


    @pyqtSlot()
    def _exportSettingsChanged(self):
        self.exportButton.setEnabled(False)
        self.processButton.setEnabled(True)

    def _load_file(self, fileName):
        settings = QSettings()
        # Check that file is not a broken link
        fileName = os.path.normpath(fileName)
        if os.path.isfile(fileName):
            lastWD, filename = os.path.split(fileName)
            # Change system working dir to target's dir
            os.chdir(lastWD)
            settings.setValue("General/lastWD", lastWD)
            settings.sync()
            with open(filename, 'r') as f:
                txt = f.read()
                self.sourceText.setPlainText(txt)
                self.openedFilename = filename
                self.needSaving = False
                self._modify_title()
                self.on_processButton_clicked()


    def _modify_title(self):
        if self.needSaving:
            title = "PyCirkuit - {filename}*".format(filename=self.openedFilename)
        else:
            title = "PyCirkuit - {filename}".format(filename=self.openedFilename)
        self.setWindowTitle(title)


    @pyqtSlot()
    def _resize_preview(self):
        # First, retrieve the pixmap rect
        rect = self.imageViewer.getRect() 
        # Next, calculate the differences between dock widget and its contents
        extraHeight = self.previewWidget.height() - self.imageViewer.viewport().height()
        extraWidth  = self.previewWidget.width() - self.imageViewer.viewport().width()
        # Then resize the dock widget in a way that acknowledges the size constraints of other widgets
        dockWidgetLocation = self.dockWidgetArea(self.previewWidget)
        if (dockWidgetLocation == Qt.TopDockWidgetArea) or (dockWidgetLocation == Qt.BottomDockWidgetArea):
            self.resizeDocks([self.previewWidget], [rect.height() + extraHeight +2], Qt.Vertical)
        elif (dockWidgetLocation == Qt.LeftDockWidgetArea) or (dockWidgetLocation == Qt.RightDockWidgetArea):
            self.resizeDocks([self.previewWidget], [rect.width() + extraWidth +2], Qt.Horizontal)


    def _save_buffer(self,  dst):
        try:
            f = open(dst,'w', encoding='UTF-8')
            f.write(self.sourceText.toPlainText())
        except OSError as e:
            errMsg = _translate("MessageBox", "Error saving source file: ", "Error message") + e.strerror + ".\n\n"
            errMsg += _translate("MessageBox", "Cannot execute command.", "Error message")
            QtWidgets.QMessageBox.critical(self, _translate("MessageBox", "PyCirkuit - Error", "Message Box title"),  errMsg)
            return
        else:
            settings = QSettings()
            lastWD, self.openedFilename = os.path.split(dst)
            os.chdir(lastWD)
            settings.setValue("General/lastWD", lastWD)
            self.needSaving = False
            self._modify_title()
            self.actionSave.setEnabled(False)
        finally:
            f.close()


    def closeEvent(self,  event):
        if self.needSaving:
            self._ask_saving()
        super().closeEvent(event)


    @pyqtSlot()
    def on_actionAbout_triggered(self):
        """
        Slot documentation goes here.
        """
        dlg = AboutDialog()
        dlg.exec()


    @pyqtSlot()
    def on_actionCMMan_triggered(self):
        # Search for Circuit Macros PDF manual
        if self._enforce_circuit_macros():
            try:
                cmMgr = CircuitMacrosManager(self)
                cmPath = cmMgr.getManUrl()
                # Open it with the default app. We can do that using Qt or in a mode pythonic way (os.system...)
                # I choose the former because I want Qt to deal with the differences between OSes
                QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(cmPath))
            except PyCktCMManNotFoundError as error:
                QtWidgets.QMessageBox.warning(self, error.title,  str(error))


    @pyqtSlot()
    def on_actionDpicMan_triggered(self):
        try:
            dpic = ToolDpic();
            path = dpic.getManUrl()
            QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(path))
        except PyCktDocNotFoundError as err:
            msgBox = QtWidgets.QMessageBox(self)
            msgBox.setWindowTitle(err.title)
            msgBox.setIcon(QtWidgets.QMessageBox.Warning)
            msgBox.setText(str(err))
            msgBox.setInformativeText(err.moreInfo)
            msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
            msgBox.exec()


    @pyqtSlot()
    def on_actionNew_triggered(self, newName=None):
        if self.needSaving:
            self._ask_saving()
        txt = _translate("MainWindow", ".PS\nscale=2.54\ncct_init\n\nl=elen_\n# Enter your drawing code here\n.PE\n",  "Template text. Translate ONLY the commented out text (line starting with '#')")
        self.sourceText.setText(txt)
        if newName == None:
            self.openedFilename = self._translatedUnnamed
            self.needSaving = False
        else:
            settings = QSettings()
            lastWD, filename = os.path.split(newName)
            # Change system working dir to target's dir
            os.chdir(lastWD)
            settings.setValue("General/lastWD", lastWD)
            settings.sync()
            self.openedFilename = filename
            self.needSaving = True
        self._modify_title()
        self.imageViewer.clearImage()


    @pyqtSlot()
    def on_actionOpen_triggered(self):
        if self.needSaving:
            self._ask_saving()
    
        # Instantiate a settings object
        settings = QSettings()

        # Show the 'load file' Dialog
        fdlg = QtWidgets.QFileDialog(self)
        fdlg.setWindowTitle(_translate("MainWindow", "Source File Selection", "File Dialog title"))
        fdlg.setDirectory(settings.value("General/lastWD",  ""))
        fdlg.setNameFilters([
            _translate("MainWindow", "PyCirkuit files (*.ckt)", "File filter text"), 
            _translate("MainWindow", "M4 macro files (*.m4)", "File filter text"),
            _translate("MainWindow", "TeX files (*.tex)", "File filter text"),
            _translate("MainWindow", "Any files (*)", "File filter text")])
        fdlg.setFileMode(QtWidgets.QFileDialog.ExistingFile)
        fdlg.setOptions(QtWidgets.QFileDialog.DontUseNativeDialog | QtWidgets.QFileDialog.ReadOnly)
        fdlg.setViewMode(QtWidgets.QFileDialog.Detail)
        fdlg.setFilter(QtCore.QDir.AllDirs | QtCore.QDir.Files | QtCore.QDir.NoDotAndDotDot)
        fitxer = ""
        if fdlg.exec():
            fitxer = fdlg.selectedFiles()[0]
        fdlg.close()

        # Check that user didn't press 'Cancel' on the Dialog Box...
        if fitxer != "":
            self._load_file(fitxer)
    
    
    @pyqtSlot()
    def on_actionPreferences_triggered(self):
        """
        Slot documentation goes here.
        """
        cfgDlg = ConfigDialog()
        cfgDlg.exportSettingsChange.connect(self._exportSettingsChanged)
        cfgDlg.exec()


    @pyqtSlot()
    def on_actionSaveAs_triggered(self):
        settings = QSettings()
        lastWD = settings.value("General/lastWD", QtCore.QStandardPaths.displayName(QtCore.QStandardPaths.HomeLocation))
        fdlg = QtWidgets.QFileDialog(self)
        fdlg.setWindowTitle(_translate("MainWindow", "Enter a file to save into", "File Dialog title"))
        fdlg.setDirectory(lastWD)
        fdlg.setFilter(QtCore.QDir.AllDirs | QtCore.QDir.Files)
        fdlg.setNameFilters([
            _translate("MainWindow", "PyCirkuit files (*.ckt)", "File filter"),
            _translate("MainWindow", "Any files (*)", "File filter")])
        fdlg.setOptions(QtWidgets.QFileDialog.DontUseNativeDialog)
        fdlg.setFileMode(QtWidgets.QFileDialog.AnyFile)
        fdlg.setViewMode(QtWidgets.QFileDialog.Detail)
        fdlg.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
        if fdlg.exec():
            dst = fdlg.selectedFiles()[0]
            self._save_buffer(dst)
        fdlg.close()


    @pyqtSlot()
    def on_actionSave_triggered(self):
        settings = QSettings()
        lastWD = settings.value("General/lastWD", "")
        filePath = os.path.join(lastWD , self.openedFilename)
        if os.path.isfile(filePath):
            self._save_buffer(filePath)
        else:
            self.actionSaveAs.trigger()


    @pyqtSlot()
    def on_exportButton_clicked(self):
        if self.openedFilename == self._translatedUnnamed:
            msgBox = QtWidgets.QMessageBox(self)
            msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Warning",  "Message Box title"))
            msgBox.setIcon(QtWidgets.QMessageBox.Warning)
            msgBox.setText(_translate("MessageBox", "The source file isn't saved yet.", "Message box text."))
            msgBox.setInformativeText(_translate("MessageBox", "Please save the source file somewhere prior to exporting it.",  "Message Box text"))
            msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
            response = msgBox.exec()
            return
        # TODO: The "src" file should be provided by the processor class upon requested
        settings = QSettings()
        lastWD = settings.value("General/lastWD")
        toSave = []
        if settings.value("Export/exportTIKZ", type=bool):
            toSave.append("tikz")
        if settings.value("Export/exportSVG", type=bool):
            toSave.append("svg")
        if settings.value("Export/exportPDF", type=bool):
            toSave.append("pdf")
        if settings.value("Export/exportPNG", type=bool):
            toSave.append("png")
        if settings.value("Export/exportJPEG", type=bool):
            toSave.append("jpeg")
        for fileType in toSave:
            src = os.path.join(pycirkuit.__tmpDir__.path(), "cirkuit_tmp") + os.extsep + fileType
            dst = os.path.join(lastWD, os.path.splitext(self.openedFilename)[0]) + os.extsep + fileType
            try:    
                if os.path.exists(dst):
                    msgBox = QtWidgets.QMessageBox(self)
                    msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Warning",  "Message Box title"))
                    msgBox.setIcon(QtWidgets.QMessageBox.Warning)
                    msgBox.setText(_translate("MessageBox", "There's already a file named \"{filename}\" at working directory.", "Message box text. Don't translate '{filename}'").format(filename=self.openedFilename.partition('.')[0]+os.extsep+fileType))
                    msgBox.setInformativeText(_translate("MessageBox", "Do you want to overwrite it?",  "Message Box text"))
                    msgBox.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
                    saveAsButton = msgBox.addButton(_translate("MessageBox", "Save As...",  "Button text"),  QtWidgets.QMessageBox.AcceptRole)
                    msgBox.setDefaultButton(QtWidgets.QMessageBox.No)
                    response = msgBox.exec()
                    # Overwrite
                    if response == QtWidgets.QMessageBox.Yes:
                        copyfile(src, dst)
                        self.exportButton.setEnabled(False)
                    # Save with another name (and ask for it first)
                    if (response == QtWidgets.QMessageBox.NoButton) and (msgBox.clickedButton() == saveAsButton):
                        if self._ask_export_as(src, dst):
                            self.exportButton.setEnabled(False)
                    # Any other option means user doesn't want to overwrite the file -> Exit
                else:
                    copyfile(src, dst)
                    self.exportButton.setEnabled(False)
            except PermissionError as err:
                msgBox = QtWidgets.QMessageBox(self)
                msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
                msgBox.setIcon(QtWidgets.QMessageBox.Critical)
                msgBox.setText(_translate("MessageBox", "Permission denied writing the file {filename}.", "Message box text. Don't translate '{filename}'").format(filename=err.filename))
                msgBox.setInformativeText(_translate("MessageBox", "Please try to export again with another name and/or location.",  "Message Box text"))
                msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
                response = msgBox.exec()
                if self._ask_export_as(src, dst):
                    self.exportButton.setEnabled(False)
            except OSError:
                msgBox = QtWidgets.QMessageBox(self)
                msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
                msgBox.setIcon(QtWidgets.QMessageBox.Critical)
                msgBox.setText(_translate("MessageBox", "An error has occurred trying to export the file. The error says:", "Message Box text"))
                msgBox.setInformativeText(err.strerror)
                msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
                response = msgBox.exec()


    @pyqtSlot()
    def on_processButton_clicked(self):
        def writeHeader(tool):
            aux = header.format(toolLongName=self.extTools[tool].longName)
            self.outputText.appendPlainText('\n' + aux)
            self.outputText.appendPlainText("="*len(aux))
                
        def writeOk():
            self.outputText.appendPlainText(_translate("OutputLog", " + No execution errors", "Output log info"))

        # STEP 0: Basic checks for the existence of auxiliary programs/utilities
        # Check if we have all the auxiliary apps correctly installed
        if not self._check_programs():
            return
        # Check if we have the Circuit Macros at the folder specified into the Settings
        if not self._enforce_circuit_macros():
            return
        # Check if the template file exists and is valid
        if not self._check_templates():
            return

        try:
            settings = QSettings()
            # STEP 1: Prepare the Progress Bar
            self.sbProgressBar.setRange(0, 8)
            self.sbProgressBar.setValue(0)
            self.sbProgressBar.setVisible(True)

            # STEP 2: Save current WD and set a new one
            savedWD = os.getcwd()
            os.chdir(pycirkuit.__tmpDir__.path())
            
            # STEP 3: Establish a temporary file base name to store intermediate results
            tmpFileBaseName = "cirkuit_tmp"
            with open("{baseName}.ckt".format(baseName=tmpFileBaseName), 'w') as tmpFile:
                tmpFile.write(self.sourceText.toPlainText())
            
            # STEP 4: Change cursor shape temporarily
            app = QtWidgets.QApplication.instance()
            app.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
            
            # STEP 5: Clear log text
            self.outputText.clear()
            self.outputText.setPlainText(_translate("OutputLog", ">>>>> Start processing", "Output log info"))
            header = _translate("OutputLog", "Output of {toolLongName}:", "Output log info. Do NOT modify/translate the '{toolLongName}' variable")
 
            # STEP 6: Call M4: .CKT -> .PIC
            self.statusBar.showMessage(_translate("StatusBar", "Converting: Circuit Macros -> PIC", "Status Bar message"))
            writeHeader(ToolM4)
            self.extTools[ToolM4].execute(tmpFileBaseName)
            writeOk()
            self.sbProgressBar.setValue(1)

            # STEP 7a: Call dpic: .PIC -> .TIKZ
            self.statusBar.showMessage(_translate("StatusBar", "Converting: PIC -> TIKZ", "Status Bar message"))
            writeHeader(ToolDpic)
            self.extTools[ToolDpic].execute(tmpFileBaseName, outputType = pycirkuit.Option.TIKZ)
            writeOk()
            self.sbProgressBar.setValue(2)

            # STEP 7b: Call dpic: .PIC -> .SVG
            if settings.value("Export/exportSVG", type=bool):
                self.statusBar.showMessage(_translate("StatusBar", "Converting: PIC -> SVG", "Status Bar message"))
                writeHeader(ToolDpic)
                self.extTools[ToolDpic].execute(tmpFileBaseName, outputType = pycirkuit.Option.SVG)
                writeOk()
                self.sbProgressBar.setValue(3)

            # STEP 8: Call PDFLaTeX: .TIKZ -> .PDF
            # First we have to embed the .TIKZ code inside a suitable template
            self.statusBar.showMessage(_translate("StatusBar", "Converting: TIKZ -> PDF", "Status Bar message"))
            writeHeader(ToolPdfLaTeX)
            self.extTools[ToolPdfLaTeX].execute(tmpFileBaseName)
            writeOk()
            self.sbProgressBar.setValue(4)

            # STEP 9: Call pdftoppm to convert the PDF into a bitmap image to visualize it: .PDF -> .PNG
            self.statusBar.showMessage(_translate("StatusBar", "Converting: PDF -> PNG", "Status Bar message"))
            writeHeader(ToolPdfToPng)
            settings.beginGroup("Export")
            dpi = settings.value("exportDPI", type=int)
            copyfile(tmpFileBaseName+os.extsep+"pdf", tmpFileBaseName+"_display"+os.extsep+"pdf")
            self.extTools[ToolPdfToPng].execute(tmpFileBaseName+"_display", resolution=150)
            if settings.value("exportPNG", type=bool):
                self.extTools[ToolPdfToPng].execute(tmpFileBaseName, resolution=dpi)
            if settings.value("exportJPEG", type=bool):
                q = settings.value("exportQuality", type=int)
                self.extTools[ToolPdfToJpeg].execute(tmpFileBaseName, resolution=dpi, quality=q)
            settings.endGroup()
            writeOk()
            self.sbProgressBar.setValue(5)
            
            # STEP 10: Call pdftoppm to convert the PDF into a JPEG
            settings.beginGroup("Export")
            if settings.value("exportJPEG", type=bool):
                dpi = settings.value("exportDPI", type=int)
                q = settings.value("exportQuality", type=int)
                self.statusBar.showMessage(_translate("StatusBar", "Converting: PDF -> JPEG", "Status Bar message"))
                writeHeader(ToolPdfToPng)
                self.extTools[ToolPdfToJpeg].execute(tmpFileBaseName, resolution=dpi, quality=q)
                writeOk()
                self.sbProgressBar.setValue(6)
            settings.endGroup()
            
            # STEP 10: Visualize the image (can fail)
            self.sbProgressBar.setValue(7)
            self.imageViewer.setImage(tmpFileBaseName+"_display", adjustIGU=True)

        except PyCktToolExecutionError as err:
            self.imageViewer.setText(_translate("MainWindow", "Error!", "Fallback text to be displayed when the image cannot be generated"))
            self.outputText.appendPlainText(err.moreInfo)
            if err.tool == ToolPdfLaTeX:
                with open(tmpFileBaseName+'.log', 'rt') as f:
                    for line in f.readlines():
                        self.outputText.appendPlainText(line)
            self._display_error(err)
        except PyCktImageError as err:
            self.imageViewer.setText(_translate("MainWindow", "Error!", "Fallback text to be displayed when the image cannot be generated"))
            self._display_error(err)
        else:
            # If all went well and we have a generated image, we can 
            self.processButton.setEnabled(False)
            self.exportButton.setEnabled(True)
        finally:
            os.chdir(savedWD)
            self.statusBar.showMessage("")
            self.sbProgressBar.setVisible(False)
            app.restoreOverrideCursor()


    @pyqtSlot()
    def on_sourceText_textChanged(self):
        self.outputText.clear()
        if self.insideConstructor:
            return
        else:
            self.needSaving = True
            self._modify_title()
            self.actionSave.setEnabled(True)
            self.processButton.setEnabled(True)
            self.exportButton.setEnabled(False)



# -*- coding: utf-8 -*-
"""
Module implementing a class to handle the pdflatex external tool
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import re

# Third-party imports
from PyQt5.QtCore import QCoreApplication,  QSettings

# Local application imports
from pycirkuit.tools.tool_base import ExternalTool, PyCktToolExecutionError

# Translation function
_translate = QCoreApplication.translate

class ToolPdfLaTeX(ExternalTool):
    # Class variable
    ID = 'PDFLATEX'
    def __init__(self):
        super().__init__("pdflatex", _translate("ExternalTool", "pdfLaTeX program", "Tool Long Name"))
        
    def execute(self, baseName):
        # Calculate src and dst names
        tikz = baseName + '.tikz'
        tex = baseName + '.tex'
        # Instantiate a settings object to load config values. At this point the config have valid entries, so don't test much
        settings = QSettings()
        latexTemplateFile = settings.value("General/templatePath")
        # Now we read a LaTeX template and wrap the tikz code inside
        templateCode = ""
        with open("{templateFile}".format(templateFile=latexTemplateFile), 'r') as template:
            templateCode = template.read()        
        with open("{source}".format(source=tikz), 'r') as f, \
             open('{destination}'.format(destination=tex), 'w') as g:
            source = f.read()
            dest = templateCode.replace('%%SOURCE%%', source, 1)
            g.write(dest)
            g.write('\n')
        # Execution of pdfLaTeX creates a PDF file
        command = [self.executableName, "-interaction=batchmode", "-halt-on-error", "{texFile}".format(texFile=tex)]
        errMsg = _translate("ExternalTool", "PDFLaTeX: Error converting TIKZ -> PDF", "Error message")
        try:
            super().execute(command, errMsg)
        except PyCktToolExecutionError as err:
            # If a LaTeX error has been triggered, try to obtain a meaningful error message
            # Very useful: https://regex101.com/#python
            info = ""
            with open("{basename}.log".format(basename=baseName),'r') as log:
                expr = "! (\S.*$)|l\.([0-9]+) (.*$)"
                prog = re.compile(expr)
                for line in log:
                    # 'match' searches from line beginning
                    match = prog.match(line)
                    if match:
                        if match.group(0)[0] == 'l':
                            #TODO: Perhaps we can return some info (error string?) to allow spotting the error at the source file
                            info = _translate("ExternalTool", "Error at TeX file line number {N}".format(N=match.group(2)), "Error message. Don't translate '{N}'") + '\n'
                            info += match.group(3) + '\n'
                        else:
                            info += match.group(1) + '\n'
            err.moreInfo = info
            raise err



# -*- coding: utf-8 -*-
"""
Module implementing the functions to run when a command option is called
"""
# Copyright (C) 2018-2019 Orestes Mas
# Copyright (C) 2019 Aniol Marti
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import sys
import glob
from os.path import abspath, realpath, isfile, isdir
from subprocess import run
from enum import Enum

# Third-party imports
from PyQt5.QtCore import QObject, QCoreApplication, QCommandLineParser, QCommandLineOption, QSettings

# Local imports
import pycirkuit
from pycirkuit.exceptions import *
from pycirkuit import Option, __productname__
from pycirkuit.tools.processor import PyCirkuitProcessor, Overwrite

# Translation function
_translate = QCoreApplication.translate

# A Enumeration class
class Decision(Enum):
    ABORT = 1
    SKIP = 2
    OPEN = 3

# The command line parser (Qt-based)
class PyCirkuitParser(QObject):
    def _initStrings(self):
        self._appDescriptionStr = _translate("CommandLine",
            "\n"
            "{productName} is a front-end for Circuit Macros by Dwight Aplevich,\n"
            "which are a set of macros for drawing high-quality line diagrams\n"
            "to be included in TeX, LaTeX, web or similar documents.",
            "Commandline application description. Don't translate '{productName}'.")
        self._batchOptionStr = _translate("CommandLine",
            "Activates batch (unattended) mode, and convert files specified by <path> to {formatID} format. Several output formats can be used together.", 
            "Commandline option description. Don't translate the '{formatID}' variable.")
        self._dpiOptionStr = _translate("CommandLine",
            "Sets the resolution of output raster images (png, jpg), in dots per inch. Value <N> is mandatory. If option is not set, default is {defaultDPI}dpi (defined in 'settings' dialog).", 
            "Commandline argument description. Don't translate the '{defaultDPI}' variable.")
        self._qualityOptionStr = _translate("CommandLine",
            "Sets the quality of output raster lossy images (jpg), in percent. Value <Q> is mandatory. If option is not set, default is {defaultQuality}% (defined in 'settings' dialog).", 
            "Commandline option description. Don't translate the '{defaultQuality}' variable.")
        self._overwriteOptionStr = _translate("CommandLine", 
            "Overwrite by default the converted files if they are present. If not set, user will be asked at runtime.", 
            "Commandline option description.")
        self._followLinksOptionStr = _translate("CommandLine", 
            "Follow symbolic links.\n"
            "- If not set, destination file will be saved into the same directory where the source file is, whether it's a real file or a symbolic link.\n"
            "- If set, destination file will be saved into the directory where the real source file is located.", 
            "Commandline option description.")
        self._destDirOptionStr = _translate("CommandLine", 
            "Save all converted files into the same destination directory <D> (value is mandatory).", 
            "Commandline option description. Don't translate '<D>'.")
        self._recurseOptionStr = _translate("CommandLine",
            "Using this option the pattern '**' will match any files and zero or more subdirs, so '**/*.ckt' will match all files with 'ckt' extension in the current directory and all its subdirectories.", 
            "Commandline option description.")
        self._pathDescriptionStr = _translate("CommandLine", 
            "Path to source drawing file(s) to process. Wildcards accepted.\n"
            "- If no <path> is given, the GUI is opened.\n"
            "- If <path> points to only one file and no batch conversion options are present, this file is opened into the GUI for editing.\n"
            "- If <path>s point to more than one valid file and a combination of output formats options are present, these source files are processed sequentially in batch (unattended) mode and converted into the requested formats.\n"
            "- Specifying more than one file to process with no output format options present is not allowed.",
            "Commandline argument description. If you translate <path>, translate the name and path syntax accordingly.")
        self._seeHelpStr = _translate("CommandLine",
            'Try "{appName} --help" to get more information.',
            "Command-line error message. Don't translate '{appName}'.")

    def __init__(self, args):
        super().__init__()
        self.args = args
        settings = QSettings()
        self.imageParam = {
            Option.DPI: settings.value("Export/exportDPI", 150, type=int),  
            Option.QUAL: settings.value("Export/exportQuality", 80, type=int), 
        }
        self.cli_mode = False
        self.requestedRecursive = False
        self.overwrite = Overwrite.UNSET
        self.dstDir = ""
        self._initStrings()
        self.parser = QCommandLineParser()
        #self.parser.setSingleDashWordOptionMode(QCommandLineself.parser.ParseAsLongOptions)
        self.parser.setApplicationDescription(self._appDescriptionStr.format(productName=__productname__))
        # Adding the '-h, --help' option
        self.parser.addHelpOption()
        # Adding the '-v --version' option
        self.parser.addVersionOption()
        # Allowing positional arguments (the file/files to open or process)
        self.parser.addPositionalArgument(
            _translate("CommandLine", "path", "Commandline argument name. If you translate this name, translate it accordingly into the path description and path syntax."), 
            self._pathDescriptionStr, 
            _translate("CommandLine",  "[<path> [ <path2>...]]",  "Commandline argument syntax. If you translate <path>, translate the name and path description accordingly.")
        )
        # Adding command line options
        self.options = {
            Option.TIKZ: QCommandLineOption(
                    ["t", "tikz"],
                    self._batchOptionStr.format(formatID='TIkZ'), 
                ),
            Option.PDF:  QCommandLineOption(
                    ["f", "pdf"],
                    self._batchOptionStr.format(formatID='PDF'),
                ),
            Option.PNG:  QCommandLineOption(
                    ["p", "png"],
                    self._batchOptionStr.format(formatID='PNG'),
                ),
            Option.JPEG: QCommandLineOption(
                    ["j", "jpeg"],
                    self._batchOptionStr.format(formatID='JPEG'),
                ),
            Option.SVG: QCommandLineOption(
                    ["s", "svg"],
                    self._batchOptionStr.format(formatID='SVG'),
                ),
            Option.DPI:  QCommandLineOption(
                    ["dpi"], 
                    self._dpiOptionStr.format(defaultDPI=self.imageParam[Option.DPI]),
                    "N", 
                ), 
            Option.QUAL: QCommandLineOption(
                    ["quality"], 
                    self._qualityOptionStr.format(defaultQuality=self.imageParam[Option.QUAL]), 
                    "Q", 
                ), 
            Option.DEST: QCommandLineOption(
                    ["destination"], 
                    self._destDirOptionStr, 
                    "D", 
                ), 
            Option.LINK: QCommandLineOption(
                    ["links"], 
                    self._followLinksOptionStr, 
                ), 
            Option.OVER: QCommandLineOption(
                    ["overwrite"], 
                    self._overwriteOptionStr, 
                ), 
            Option.REC:  QCommandLineOption(
                    ["r"],
                    self._recurseOptionStr, 
                ),
        }
        # Adding the options in the list
        for option in self.options.values():
            self.parser.addOption(option)

    def _checkFiles(self):
        # Test for some path passed as parameters, or none
        paths = self.parser.positionalArguments()
        # User gave some filespec to process?
        pathPresent = (len(paths) > 0)
        # User may have entered more than one path, and these can contain wildcards
        # We have to expand them into files prior to process
        self.requestedFilesToProcess = list()
        for pathSpec in paths:
            for f in glob.iglob(pathSpec, recursive=self.requestedRecursive):
                if isfile(realpath(f)):
                    f = realpath(f) if self.followSymlinks else abspath(f)
                    self.requestedFilesToProcess.append(f)
        NumFiles = len(self.requestedFilesToProcess)
        if (NumFiles == 0) and pathPresent:
            print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "The given path does not match any existing file.", "Commandline error message"))
            print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
            sys.exit(-1)
        return NumFiles

    def _checkFileOptions(self):
        self.followSymlinks = self.parser.isSet(self.options[Option.LINK])
        self.overwrite = Overwrite.ALL if self.parser.isSet(self.options[Option.OVER]) else Overwrite.UNSET
        if self.parser.isSet(self.options[Option.DEST]):
            self.dstDir = abspath(self.parser.value((self.options[Option.DEST])))
            if not isdir(self.dstDir):
                print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "The given path does not match any existing file.", "Commandline error message"))
                print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
                sys.exit(-1)
        else:
            self.dstDir = ""
    
    def _checkFormats(self):
        # Test for requested output formats. If any, cli_mode is set.
        validOutputFormats = {Option.TIKZ, Option.PNG, Option.PDF, Option.JPEG, Option.SVG}
        self.requestedOutputFormats = set()
        for outputFormat in validOutputFormats:
            if self.parser.isSet(self.options[outputFormat]):
                self.cli_mode = True
                self.requestedOutputFormats.add(outputFormat)
        NumFormats = len(self.requestedOutputFormats)
        return NumFormats

    def _checkRecursive(self):
        self.requestedRecursive = self.parser.isSet(self.options[Option.REC])
    
    def _checkRasterOptions(self):
        # Set the "dpi" and "quality" parameters to the values provided by user, if any
        # process the "dpi" option
        if self.parser.isSet(self.options[Option.DPI]):
            try:
                self.imageParam[Option.DPI] = int(self.parser.value(self.options[Option.DPI]))
                if self.imageParam[Option.DPI] not in range(25, 3001):
                    raise Exception()
            except:
                print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "The --dpi parameter must be an integer between 25 and 3000.", "Error message"))
                print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
                sys.exit(-1)
        # Process the "quality" option
        if self.parser.isSet(self.options[Option.QUAL]):
            try:
                self.imageParam[Option.QUAL] = int(self.parser.value(self.options[Option.QUAL]))
                if self.imageParam[Option.QUAL] not in range(0, 101):
                    raise Exception()
            except:
                print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "The --quality parameter must be an integer between 0 and 100.", "Error message"))
                print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
                sys.exit(-1)

    def parseCmdLine(self):
        self.parser.process(self.args)
        ##### FETCH OPTIONS AND ARGUMENTS
        # Test if "recursive" flag is set
        self._checkRecursive()
        # Check if user set some image raster parameters
        self._checkRasterOptions()
        # Test how many output formats were requested.
        self._checkFormats()
        # Test for various file options
        self._checkFileOptions()
        # Find files to process
        NumFiles = self._checkFiles()
        
        ##### Process CLI mode
        if self.cli_mode:
            # Is an error to call pycirkuit with a batch option and no filenames
            if (NumFiles == 0):
                print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "Batch processing requested with no files.", "Commandline error message"))
                print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
                sys.exit(-1)
            # Instantiate a processor object for this CLI session
            try:
                processor = PyCirkuitProcessor()
                for fileName in self.requestedFilesToProcess:
                    processed = False
                    print(_translate("CommandLine",  "Processing file:", "Command line message. Will be followed by an absolute file path"), fileName)
                    while not processed:
                        processor.beginProcessing(fileName)
                        for format in self.requestedOutputFormats:
                            try:
                                processor.requestResult(format, dstDir=self.dstDir, overwrite=self.overwrite, dpi=self.imageParam[Option.DPI], quality=self.imageParam[Option.QUAL])
                            except PyCktToolExecutionError as err:
                                answer = self._askOnError(err)
                                if answer == Decision.ABORT:
                                    # Terminate program
                                    sys.exit(-1)
                                elif answer == Decision.SKIP:
                                    # Consider file processed and jump to the next one
                                    processed = True
                                    continue
                                elif answer == Decision.OPEN:
                                    # Open a IGU
                                    try:
                                        run(["pycirkuit", fileName], shell=False, check=False)
                                        # Breaking here puts us out of the "for" loop and source is re-read in "beginProcesing"
                                        break
                                    except:
                                        processed = True
                        else:
                            processed = True
                    print("")
            except PyCirkuitError as err:
                print("\npycirkuit:", err)
                sys.exit(-1)
            print(_translate("CommandLine", "Files processed: {N}.", "Command line message. {N} will be an integer, don't translate it.").format(N=NumFiles))
            sys.exit(0)

        ##### Process GUI mode. Perform some final checks and exit.
        if NumFiles == 0:
            return None
        elif NumFiles == 1:
            return abspath(self.requestedFilesToProcess[0])
        else:
            print(QCoreApplication.applicationName() + ": " + _translate("CommandLine", "More than one file to process with no batch option given.", "Commandline error message"))
            print(self._seeHelpStr.format(appName=QCoreApplication.applicationName()))
            sys.exit(-1)

    def _askOnError(self, err):
        # Test if we have GUI or not, and ask accordingly
        print("\npycirkuit:", err)
        question = _translate("CommandLine-UserInput2", 
            "Please choose what to do: [a]bort processing, [s]kip file, [o]pen in GUI for editing: ", 
            "WARNING!! Critical translation. You should translate this message to your language, enclosing into brackets one single DIFFERENT character for each option, and translate accordingly the characters in the next message.")
        answerAbort = _translate("CommandLine-UserInput2",
            "a", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Please choose what to do:'")
        answerSkip = _translate("CommandLine-UserInput2",
            "s", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Please choose what to do:'")
        answerOpen = _translate("CommandLine-UserInput2",
            "o", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Please choose what to do:'")
        if pycirkuit.__haveGUI__:
            question = _translate("CommandLine-UserInput2", 
                "Please choose what to do: [a]bort processing, [s]kip file, [o]pen in GUI for editing: ", 
                "WARNING!! Critical translation. You should translate this message to your language, enclosing into brackets one single DIFFERENT character for each option, and translate accordingly the characters in the next message.")
            answers = {
                answerAbort: Decision.ABORT, 
                answerSkip: Decision.SKIP, 
                answerOpen: Decision.OPEN
            }
        else:
            question = _translate("CommandLine-UserInput2", 
                "Please choose what to do: [a]bort processing, [s]kip file: ", 
                "WARNING!! Critical translation. You should translate this message to your language, enclosing into brackets one single DIFFERENT character for each option, and translate accordingly the characters in the next message.")
            answers = {
                answerAbort: Decision.ABORT, 
                answerSkip: Decision.SKIP, 
            }
        # Ask for user decision, loop until answered
        while True:
            answer = input(question)
            if answer.lower() in answers:
                return answers[answer]



# -*- coding: utf-8 -*-
"""
Module implementing a class to handle the dpic external tool
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
from os.path import *

# Third-party imports
from PyQt5.QtCore import QCoreApplication, QStandardPaths

# Local application imports
from pycirkuit import Option
from pycirkuit.tools.tool_base import ExternalTool, PyCktDocNotFoundError

# Translation function
_translate = QCoreApplication.translate

class ToolDpic(ExternalTool):
    # Class variable
    ID = 'DPIC'
    def __init__(self):
        super().__init__("dpic", _translate("ExternalTool", "'PIC' language compiler", "Tool Long Name"))
        
    def execute(self, baseName,  outputType = Option.TIKZ):
        # Calculate src and dst names
        src = baseName + '.pic'
        if outputType == Option.TIKZ:
            dst = baseName + '.tikz'
            flag = '-g'
        elif outputType == Option.SVG:
            dst = baseName + '.svg'
            flag = '-v'
        #NOTE: We add the '-z' flag to always execute dpic in "safe mode"
        # Instantiate a settings object to load config values. At this point the config have valid entries, so don't test much
        command = [self.executableName, flag, '-z', "{source}".format(source=src)]
        errMsg = _translate("ExternalTool", "DPIC: Error processing source file", "Error message")
        super().execute(command, errMsg, destination=dst)
        
    def getManUrl(self):
        import glob
        try:
            import magic
        except ImportError:
            raise PyCirkuitError(_translate("ExternalTool", "Module 'python-magic' not found. Please check that all PyCirkuit dependencies are correctly installed.",  "Error message"))
        mime = magic.Magic(mime=True)   # Prepare to detect a file's mimetype based on its contents
        # Get standard locations for documentation in a platform-independent way
        dirList = QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
        # Append specific app subdir to each possible location
        dirList = [normpath(join(dir, "doc", "dpic")) for dir in dirList]
        # Explore the generated list searching for pdf or compressed pdf
        for testPath in dirList:
            # Perhaps we should search for *.pdf* or, at least, for *.pdf AND *.pdf.gz
            candidates = glob.glob(join(testPath, "dpic-doc.pdf"))
            candidates.extend(glob.glob(join(testPath, "dpic-doc.pdf.gz")))
            for candidate in candidates:
                mimeType = mime.from_file(candidate)
                if (mimeType == "application/pdf") or (mimeType == "application/gzip"):
                    return(candidate)
        else:
            err = PyCktDocNotFoundError("dpic")
            # Add the list of possible locations to 'moreinfo' message's box variable
            info = err.moreInfo + '\n'.join(dirList)
            err.moreInfo = info
            raise err




# -*- coding: utf-8 -*-
"""
Module implementing a personalized AboutDialog.
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Third-party imports
from PyQt5.QtWidgets import QDialog

# Local application imports
from pycirkuit import __version__, __copyright__
from .Ui_aboutdialog import Ui_AboutDialog


class AboutDialog(QDialog, Ui_AboutDialog):
    """
    Class documentation goes here.
    """
    def __init__(self, parent=None):
        """
        Constructor
        
        @param parent reference to the parent widget
        @type QWidget
        """
        super(AboutDialog, self).__init__(parent)
        self.setupUi(self)
        # Now change the placeholders in dialog
        # Cannot use str.format() here because the HTML string does contain other items between curly brackets
        # Try to change the variable in the text for the version number
        s = self.textVersion.toHtml()
        self.textVersion.setHtml(s.replace('{versionNumber}', __version__, 1))
            
        # Try to change the variable in the text for the copyright
        s = self.textLicense.toHtml()
        self.textLicense.setHtml(s.replace('{copyrightInfo}', __copyright__, 1))
        
        # Start with first Tab visible
        self.tabWidget.setCurrentIndex(0)
